/* jshint globalstrict:true */
/* globals wcSettings, wc_add_to_cart_params, woocommerce_params */
'use strict';

import { changeDpiDataUrl } from 'https://unpkg.com/changedpi@1.0.4/src/index.js';

document.addEventListener('DOMContentLoaded', function() {
    document
        .querySelectorAll('form.cart')
        .forEach(formElement => formElement.addEventListener('submit', submitCartForm));
});

function addOverlay() {
    const overlay = document.createElement('div');
    overlay.id = 'locrmaps-add-to-cart-overlay';
    overlay.style.position = 'fixed';
    overlay.style.top = '0';
    overlay.style.left = '0';
    overlay.style.width = '100%';
    overlay.style.height = '100%';
    overlay.style.backgroundColor = 'rgba(0, 0, 0, 0.5)';
    overlay.style.zIndex = '1000';
    document.body.appendChild(overlay);
}

/**
 * @param {L.Map} map
 * @param {HTMLFormElement} formElement
 * @returns {Promise<FormData>}
 */
function buildFormData(map, formElement) {
    return new Promise((resolve, reject) => {
        const formData = new FormData();
        formData.append('action', 'add_custom_product_to_cart');
        const inputProductId = formElement.querySelector('input[name="product_id"]');
        if (inputProductId instanceof HTMLInputElement) {
            formData.append('product_id', inputProductId.value);
        } else {
            const addToCartButton = formElement.querySelector('button[name="add-to-cart"]');
            if (addToCartButton instanceof HTMLButtonElement) {
                formData.append('product_id', addToCartButton.value);
            }
        }
        const inputQuantity = formElement.querySelector('input[name="quantity"]');
        if (inputQuantity instanceof HTMLInputElement) {
            formData.append('quantity', inputQuantity.value);
        }
        const inputVariationId = formElement.querySelector('input[name="variation_id"]');
        if (inputVariationId instanceof HTMLInputElement) {
            formData.append('variation_id', inputVariationId.value);
        }

        const dataProductVariationsAttr = formElement.attributes.getNamedItem('data-product_variations');
        if (dataProductVariationsAttr instanceof window.Attr) {
            formData.append('product_variations', dataProductVariationsAttr.value);
        }

        const locrMap = document.querySelector('locr-map');
        if (locrMap) {
            /**
             * Hopefully an always working workaround for Safari,
             * which does not always displays all MapOverlay Elements.
             */
            const ua = navigator.userAgent.toLowerCase();
            const printFormatSize = locrMap.getPrintFormat().getSize();
            if (ua.indexOf('safari') !== -1 && ua.indexOf('chrome') === -1) {
                locrMap.getPrintCanvas().then(() => {
                    locrMap.getPrintCanvas().then(mapCanvas => {
                        formData.append('custom_image_data', changeDpiDataUrl(mapCanvas.toDataURL('image/jpeg'), printFormatSize.dpi));
                        resolve(formData);
                    }).catch(reject);
                }).catch(reject);
            } else {
                locrMap.getPrintCanvas().then(mapCanvas => {
                    formData.append('custom_image_data', changeDpiDataUrl(mapCanvas.toDataURL('image/jpeg'), printFormatSize.dpi));
                    resolve(formData);
                }).catch(reject);
            }
        }
    });
}

function removeOverlay() {
    const overlay = document.getElementById('locrmaps-add-to-cart-overlay');
    if (overlay instanceof HTMLElement) {
        document.body.removeChild(overlay);
    }
}

/**
 * @param {Event} event
 */
function submitCartForm(event) {
    const map = document.querySelector('locr-map');
    if (!map) {
        window.console.error('Map not found');
        return;
    }

    event.preventDefault();

    addOverlay();

    buildFormData(map, event.target)
        .then(async formData => {
            return fetch(woocommerce_params.ajax_url, {
                method: 'POST',
                body: formData,
            }).then(res => {
                if (!res.ok) {
                    let errorMessage = 'Failed to add product to cart (!res.ok)';
                    if (res.statusText) {
                        errorMessage += ': ' + res.statusText;
                    }
                    return Promise.reject(errorMessage);
                }

                return res.json();
            }).then(json => {
                if (!json.success) {
                    return Promise.reject('Failed to add product to cart (!json.success)');
                }

                if (window.wcSettings !== undefined && wcSettings.storePages?.cart?.permalink) {
                    window.location.href = wcSettings.storePages.cart.permalink;
                } else if (window.wc_add_to_cart_params !== undefined && wc_add_to_cart_params.cart_url) {
                    window.location.href = wc_add_to_cart_params.cart_url;
                }
            }).catch(err => window.alert(err));
        })
        .finally(removeOverlay);
}
